
import 'dart:async';
import 'package:datadome_flutter_dio/datadome_logger.dart';
import 'package:datadome_flutter_dio/tracking/event_tracker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'datadome_interceptor.dart';

class DataDomeDio {
  static const MethodChannel _channel =
      const MethodChannel('datadome_dio');
  static String lastCookieValue = "";

  /// Store DataDome cookie. The new value should be a string representing the cookie with its value and its attributes.
  /// Example: "datadome=xxxx; Max-Age=31536000; Domain=.yourdomain.com; Path=/; Secure; SameSite=Lax"
  static Future<Null> storeCookie(String cookie) async {
    if(!cookie.startsWith("datadome=") || cookie == lastCookieValue) {
      return null;
    }
    lastCookieValue = cookie;
    final prefs = await SharedPreferences.getInstance();
    prefs.setString('co.datadome.cookie', cookie);
    try {
      _channel.invokeMethod('storeCookie', { 'cookie': cookie });
    } on PlatformException catch (e) {
      DataDomeLogger.error("Error while sending message to native '${e.message}'");
    }
    sendMobileSignals();
  }

  /// Returns the DataDome cookie. The returned value should be a string representing the cookie with the datadome name and its value.
  /// Example: "datadome=xxxx"
  static Future<String> getCookie() async {
    if(lastCookieValue.isNotEmpty) {
      return "datadome=" + parseCookieValue(lastCookieValue);
    }
    final prefs = await SharedPreferences.getInstance();
    String storedCookie = prefs.getString('co.datadome.cookie')?? '';
    return "datadome=" + parseCookieValue(storedCookie);
  }
  
  static String parseCookieValue(String cookie) {
    var cookieVal = cookie.split(";").first;
    return cookieVal.split("=").last;
  }

  static void removeCookie() async{
    final prefs = await SharedPreferences.getInstance();
    prefs.remove('co.datadome.cookie');
    lastCookieValue = "";
  }

  static String getCookieDomain() {
    var lastCookieLowerCase = lastCookieValue.toLowerCase();
    if(!lastCookieLowerCase.contains("domain=")) {
      return "";
    }
    var domainSubstring = lastCookieLowerCase.split("domain=").last;
    return domainSubstring.split(";").first;
  }

  static Future<bool> sendMobileSignals() async{
    var context = DataDomeInterceptor.payloadInitializer["context"];
    if(DataDomeInterceptor.deviceInfos.isEmpty) {
      Map<String, dynamic> payloadData = await EventTracker().initPlatformState();
      payloadData["request"] = getCookieDomain();
      payloadData["ddk"] = DataDomeInterceptor.payloadInitializer["dataDomeKey"];
      payloadData["cid"] = parseCookieValue(lastCookieValue);
      MediaQueryData? media = MediaQuery.maybeOf(context);
      if(media != null) {
        payloadData["screen_x"] = media.size.width.toStringAsFixed(3);
        payloadData["screen_y"] = media.size.height.toStringAsFixed(3);
        payloadData["screen_d"] = media.devicePixelRatio;
      }
      EventTracker().setDeviceData = payloadData;
      EventTracker().sendEventPayload();
      return true;
    }
    return false;
  }
}
