
import 'package:datadome_flutter_dio/tracking/event_data.dart';
import 'package:datadome_flutter_dio/tracking/event_tracker.dart';
import 'package:flutter/material.dart';

class GestureDetection extends StatelessWidget {
  final Widget child;

  GestureDetection({required this.child}){}

  @override
  Widget build(BuildContext context) {
    final int eventInterval = 100;
    int lastTrackEventTouchDown = 0;
    int lastTrackEventTouchUp = 0;
    int lastTrackEventMove = 0;

    return GestureDetector(
      child: child,
      onTapDown: (tapDownDetails) {
        if(DateTime.now().millisecondsSinceEpoch - lastTrackEventTouchDown > eventInterval) {
          double x = tapDownDetails.globalPosition.dx;
          double y = tapDownDetails.globalPosition.dy;
          EventTracker().addEvent(getEvent(
              Events.TOUCH_DOWN, EventTracker.eventPositionTemplate(x, y)));
          lastTrackEventTouchDown = DateTime.now().millisecondsSinceEpoch;
        }
      },
      onTapUp: (tapUpDetails) {
        if(DateTime.now().millisecondsSinceEpoch - lastTrackEventTouchUp > eventInterval) {
          double x = tapUpDetails.globalPosition.dx;
          double y = tapUpDetails.globalPosition.dy;
          EventTracker().addEvent(getEvent(Events.TOUCH_UP, EventTracker.eventPositionTemplate(x, y)));
          lastTrackEventTouchUp = DateTime.now().millisecondsSinceEpoch;
        }
      },
      onHorizontalDragUpdate: (details) {
        if(DateTime.now().millisecondsSinceEpoch - lastTrackEventMove > eventInterval) {
          double x = details.globalPosition.dx;
          double y = details.globalPosition.dy;
          EventTracker().addEvent(getEvent(
              Events.TOUCH_MOVE, EventTracker.eventPositionTemplate(x, y)));
          lastTrackEventMove = DateTime.now().millisecondsSinceEpoch;
        }
      },
      onVerticalDragUpdate: (details) {
        if(DateTime.now().millisecondsSinceEpoch - lastTrackEventMove > eventInterval) {
          double x = details.globalPosition.dx;
          double y = details.globalPosition.dy;
          EventTracker().addEvent(getEvent(
              Events.TOUCH_MOVE, EventTracker.eventPositionTemplate(x, y)));
          lastTrackEventMove = DateTime.now().millisecondsSinceEpoch;
        }
      },
    );
  }

}